% result = CalculateProfiles(ass_data_fun,ass_pred_fun,p,lb,ub,data,pred,varargin)
% 
%   Function for the calculation of the prediction and validation profile
%   likelihood.
% 
%   ass_data_fun    objective function for the data
%   ass_pred_fun    objecitve function for the predictions
%   model_fun       model function
%   p               fitted parameters 
%   lb              lower boundaries of the parameter space
%   ub              upper boundaries of the parameter space
%   data            struct, measurements and design 
%   pred            struct, design for predictions

function result = CalculateProfiles(ass_data_fun,ass_pred_fun,model_fun,p,lb,ub,data,pred,varargin)

%pred = CheckDesign(pred);
%data = CheckDesign(data);

vplx = pred.y;
predfit = feval(model_fun,p,pred,varargin{:});  % predicted value 

[order,useLast] = pinitial(predfit,vplx);   % order of optimizations, good initial guess of the parameters required

A = []; B = []; Aeq = []; Beq = []; 
optimopt = optimset('Algorithm','trust-region-reflective','display','off','GradObj','on','Hessian','on','DerivativeCheck','off','TolX',0);

pests = NaN(length(vplx),length(p));
vply = NaN(length(vplx),1);
pplx = NaN(length(vplx),1);
pply = NaN(length(vplx),1);
for i=1:length(order)
    predtmp = pred;
    predtmp.y = vplx(order(i));

    if(useLast(i)==0)
        pini = p;
    else
        pini = pests(order(i-1),:);
    end
    
    [pests(order(i),:),vply(order(i))] = fmincon('ass_vpl',pini,A,B,Aeq,Beq,lb,ub,[],...
        optimopt,ass_data_fun,ass_pred_fun,model_fun,data,predtmp,varargin{:});
    
    pplx(order(i)) = feval(model_fun,pests(order(i),:),predtmp,varargin{:});  % predicted value 
    pply(order(i)) = feval('ass_vpl',pests(order(i),:),ass_data_fun,ass_pred_fun,model_fun,data,predtmp,varargin{:});
    
    %disp(['  ',num2str(i),' out of ',num2str(length(order)),' predictions done.'])
end

vplx_inter = linspace(nanmin(vplx),nanmax(vplx),1001)';
vply_inter = interp1(vplx,vply,vplx_inter,'PCHIP');
pplx_inter = linspace(nanmin(pplx),nanmax(pplx),1001)';
[dummy,ia] = unique(pplx);
pply_inter = interp1(pplx(ia),pply(ia),pplx_inter,'PCHIP');
% try
%     pply_inter = interp1(pplx,pply,pplx_inter,'cubic');
% catch % 
% end


result.p     = p;  % fitted to the data
result.pests = pests; %

result.vply  = vply;
result.vplx  = vplx;
result.vplx_inter = vplx_inter;
result.vply_inter = vply_inter;

result.pplx  = pplx;
result.pply  = pply;
result.pplx_inter = pplx_inter;
result.pply_inter = pply_inter;

result.data  = data;
result.pred  = pred;
result.lb    = lb;
result.ub    = ub;
result.varargin = varargin;
result.ass_data_fun = ass_data_fun;
result.ass_pred_fun = ass_pred_fun;
result.model_fun = model_fun;




