%%
clear all, close all
% restoredefaultpath  % matlab default path settings 

ndata   = 11;   % number of data points/time points
tmax    = 100;  % time interval  
tmin    = 0;    % time interval  
sig     = 0.1;  % noise level

p = [0.05,0.1,1];% true parameters, 1 x npara, here p(1) is the 1st rate, p(2) the 2nd, and p(3) the initial concentration of A
lb      = p/10; % lower boundaries for parameter estimation
ub      = p*10; % upper boundaries for parameter estimation

ass_data_fun    = 'ass_fun_data'; % objective function for the data
ass_pred_fun    = 'ass_fun_pred'; % objective function for the prediction
model_fun       = 'ass_ABC_ana';%'model_fun_ODE1'; % model function

pred.sigma  = 0.1;     % validation data error

data.obs = [2,3];                                       % design, which state is measured
data.t = linspace(tmin,tmax,ndata)';                % design, measurement times
data.sigma = sig;            % measurement noise level
data.f = feval(model_fun,p,data);                   % model response without noise
                                                      %data.y = data.f + randn(size(data.f)).*data.sigma;  % adding noise

data.y = [-0.080949869,NaN,0.376394515,0.205873465,0.041535718,0.212383233,-0.123839312,0.019062712,-0.006166007,0.042902675,0.03797398,...
    -0.086487992,0.151804861,0.383077418,0.666288044, 0.856965279,0.953492284,0.81653983,0.948257752,0.842301495,0.866566323,0.985895766]';

p0 = p; % initial guess for parameter estimation
%% The following two calls of fmincon are just to check whether
%% optimization works:
% 1) Try parameter estimation without prediction
[pest,ass] = fmincon(ass_data_fun,p0,[],[],[],[],lb,ub,[],...
    optimset('display','iter','Algorithm','trust-region-reflective','GradObj','on','Hessian','on','DerivativeCheck','off'),...
    model_fun,data);

% define the predicitons for which the PPL is calcualted
pred_t_vec = linspace(5,95,10);
lower_bounds = NaN(3,length(pred_t_vec));
upper_bounds = NaN(3,length(pred_t_vec));

for i=1:3
    
    pred.obs    = i;        % which state is evaluated for prediction
    
    for j = 1:length(pred_t_vec)
        pred.t      = pred_t_vec(j);       % validation/prediction design
        Preds = pred;
        Preds.y = linspace(-0.25,1.25,32)';

    % Call profile likelihood calculation
        result = CalculateProfiles(ass_data_fun,ass_pred_fun,model_fun,pest,lb,ub,data,Preds);
        [down, up] = calc_llp_bound(result.vply_inter);
        
        lower_bounds(i,j) = result.vplx_inter(down);
        upper_bounds(i,j) = result.vplx_inter(up);
    end
end
