% [f,dfdp] = model_fun_ODE1(p,data,varargin)
% 
%   Model function for reactions A->B->C
%   The model function in general calculates the model response/behaviour
%   for given paramters and design, as well as the gradient with respect to
%   parameter changes.
% 
%   data        struct, design
%   p           parameters
% 
%   f           model response
%   dfdp        ndata x npara
%               each row is the gradient w.r. to the parameters

function [f,dfdp,df2dp] = ass_ABC_ana(p,data,varargin)

x0 = p(3);
x = NaN(length(data.t),3);

x(:,1) = x0 * exp(-p(1) * data.t)';
x(:,2) = p(1) * x0 / (p(2) - p(1)) * (exp(-p(1) * data.t) - exp(-p(2) * data.t));
x(:,3) = x0 * (1+ ( p(1) * exp(-p(2) ...
                                   * data.t) - p(2) ...
                        *exp(-p(1) * data.t)) / (p(2) - p(1)));
                    
f = reshape(x(:,data.obs),size(x,1)*length(data.obs),1);

if(nargout > 1)
    dfdp = zeros(3*length(data.t),3);
    df2dp = zeros(3*length(data.t),3,3);
    
    dfdp(1:length(data.t),3) = exp(-p(1)*data.t);
    dfdp(1:length(data.t),1) = -data.t.*x0.*exp(-p(1)*data.t);
    
    dfdp(length(data.t)+1:2*length(data.t),3) = -(p(1)*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2));
    dfdp(length(data.t)+1:2*length(data.t),1) = (p(1)*x0*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2))^2 - (x0*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2)) + (p(1)*data.t.*x0.*exp(-p(1).*data.t))/(p(1) - p(2));
    dfdp(length(data.t)+1:2*length(data.t),2) = - (p(1)*x0*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2))^2 - (p(1)*data.t*x0.*exp(-p(2).*data.t))/(p(1) - p(2));
    
    dfdp(2*length(data.t)+1:end,3) = 1 - (p(1)*exp(-p(2).*data.t) - p(2)*exp(-p(1).*data.t))./(p(1) - p(2));
    dfdp(2*length(data.t)+1:end,1) = -x0*((exp(-p(2).*data.t) + p(2)*data.t.*exp(-p(1).*data.t))/(p(1) - p(2)) - (p(1)*exp(-p(2).*data.t) - p(2)*exp(-p(1).*data.t))/(p(1) - p(2))^2);
    dfdp(2*length(data.t)+1:end,2) = x0*((exp(-p(1).*data.t) + p(1)*data.t.*exp(-p(2).*data.t))/(p(1) - p(2)) - (p(1)*exp(-p(2).*data.t) - p(2)*exp(-p(1).*data.t))/(p(1) - p(2))^2);
    
    df2dp(1:length(data.t),3,1) = -data.t.*exp(-p(1).*data.t);
    df2dp(1:length(data.t),1,3) =df2dp(1:length(data.t),3,1);
    
    df2dp(1:length(data.t),1,1) = data.t.^2.*x0.*exp(-p(1)*data.t);
    
    df2dp(length(data.t)+1:2*length(data.t),3,1) = (p(1)*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2))^2 - (exp(-p(1).*data.t) - exp(-p(2).*data.t))/(p(1) - p(2)) + (p(1)*data.t.*exp(-p(1).*data.t))/(p(1) - p(2));
    df2dp(length(data.t)+1:2*length(data.t),1,3) =df2dp(length(data.t)+1:2*length(data.t),3,1);
    
    df2dp(length(data.t)+1:2*length(data.t),3,2) = - (p(1)*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2))^2 - (p(1)*data.t.*exp(-p(2).*data.t))/(p(1) - p(2));
    df2dp(length(data.t)+1:2*length(data.t),2,3) = df2dp(length(data.t)+1:2*length(data.t),3,2);
    
    df2dp(length(data.t)+1:2*length(data.t),1,1) = (2*x0*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2))^2 - (2*p(1)*x0*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2))^3 + (2*data.t.*x0.*exp(-p(1).*data.t))/(p(1) - p(2)) - (2*p(1)*data.t.*x0.*exp(-p(1).*data.t))/(p(1) - p(2))^2 - (p(1)*data.t.^2.*x0.*exp(-p(1).*data.t))/(p(1) - p(2));
    
    df2dp(length(data.t)+1:2*length(data.t),1,2) = (2*p(1)*x0*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2))^3 - (x0*(exp(-p(1).*data.t) - exp(-p(2).*data.t)))/(p(1) - p(2))^2 - (data.t.*x0.*exp(-p(2).*data.t))/(p(1) - p(2)) + (p(1)*data.t.*x0.*exp(-p(1).*data.t))/(p(1) - p(2))^2 + (p(1)*data.t.*x0.*exp(-p(2).*data.t))/(p(1) - p(2))^2;
    df2dp(length(data.t)+1:2*length(data.t),2,1) = df2dp(length(data.t)+1:2*length(data.t),1,2);
    
    df2dp(length(data.t)+1:2*length(data.t),2,2) = (p(1)*data.t.^2.*x0.*exp(-p(2)*data.t))/(p(1) - p(2)) - (2*p(1)*data.t.*x0.*exp(-p(2)*data.t))/(p(1) - p(2))^2 - (2*p(1)*x0*(exp(-p(1)*data.t) - exp(-p(2)*data.t)))/(p(1) - p(2))^3;

    
    df2dp(2*length(data.t)+1:end,3,1) = (p(1)*exp(-p(2)*data.t) - p(2)*exp(-p(1)*data.t))/(p(1) - p(2))^2 - (exp(-p(2)*data.t) + p(2)*data.t.*exp(-p(1)*data.t))/(p(1) - p(2));
    df2dp(2*length(data.t)+1:end,1,3) = df2dp(2*length(data.t)+1:end,3,1);
    
    df2dp(2*length(data.t)+1:end,3,2) = (exp(-p(1)*data.t) + p(1)*data.t.*exp(-p(2)*data.t))/(p(1) - p(2)) - (p(1)*exp(-p(2)*data.t) - p(2)*exp(-p(1)*data.t))/(p(1) - p(2))^2;
    df2dp(2*length(data.t)+1:end,2,3) = df2dp(2*length(data.t)+1:end,3,2);
    
    df2dp(2*length(data.t)+1:end,1,1) = x0*((2*(exp(-p(2)*data.t) + p(2)*data.t.*exp(-p(1)*data.t)))/(p(1) - p(2))^2 - (2*(p(1)*exp(-p(2)*data.t) - p(2)*exp(-p(1)*data.t)))/(p(1) - p(2))^3 + (p(2)*data.t.^2.*exp(-p(1)*data.t))/(p(1) - p(2)));
    
    df2dp(2*length(data.t)+1:end,1,2) = -x0*((exp(-p(1)*data.t) + p(1)*data.t.*exp(-p(2)*data.t))/(p(1) - p(2))^2 + (exp(-p(2)*data.t) + p(2)*data.t.*exp(-p(1)*data.t))/(p(1) - p(2))^2 - (2*(p(1)*exp(-p(2)*data.t) - p(2)*exp(-p(1)*data.t)))/(p(1) - p(2))^3 + (data.t.*exp(-p(1)*data.t) - data.t.*exp(-p(2)*data.t))/(p(1) - p(2)));
    df2dp(2*length(data.t)+1:end,2,1) = df2dp(2*length(data.t)+1:end,1,2);
    
    df2dp(2*length(data.t)+1:end,2,2) = -x0*((2*(p(1)*exp(-p(2)*data.t) - p(2)*exp(-p(1)*data.t)))/(p(1) - p(2))^3 - (2*(exp(-p(1)*data.t) + p(1)*data.t.*exp(-p(2)*data.t)))/(p(1) - p(2))^2 + (p(1)*data.t.^2.*exp(-p(2)*data.t))/(p(1) - p(2)));

    
    dfdp = dfdp(((min(data.obs)-1)*length(data.t))+1:max(data.obs)*length(data.t),:);
    if(isempty(find(data.obs==2,1)) && size(dfdp,1)==33)
       dfdp(12:22,:) = []; 
    end
    df2dp = df2dp(((min(data.obs)-1)*length(data.t))+1:max(data.obs)*length(data.t),:,:);
    if(isempty(find(data.obs==2,1)) && size(dfdp,1)==33)
       df2dp(12:22,:,:) = []; 
    end
end
%f = reshape(x(:,data.obs),size(x,1)*length(data.obs),1);
