% [chi2,grad,H,sens] = ass_fun_data(p,model_fun,data,varargin)
% 
%   p               parameters
%   model_fun       name of the function evaluating the model response
%   data            data struct         
%                       data.t
%                       data.y
%                       data.sigma

function [chi2,grad,H,sens] = ass_fun_data(p,model_fun,data,varargin)

if(nargout<=1)
    f = feval(model_fun,p,data,varargin{:});
    chi2 = nansum(nansum(((data.y-f)./data.sigma).^2));
elseif(nargout>1)
    [f,dfdp,df2dp] = feval(model_fun,p,data,varargin{:}); % dfdp = ndata x npara
    chi2 = nansum(nansum(((data.y-f)./(data.sigma)).^2,1),2);
    
    %sens = diag(1./data.sigma)*dfdp; % ndata x npara, sensitivities = dfdp (scaled with observational error)
    %grad = sum(-2*diag((data.y-f)./data.sigma)*sens,1); % 1 x npara, residuals are scaled with the obs. error
    sens = dfdp./data.sigma;
    grad = nansum(repmat(-2*(data.y-f)./data.sigma,1,3).*sens,1)';
    H = 2*sens'*sens + squeeze(nansum(bsxfun(@times,repmat(-2*(data.y-f)./data.sigma,1,3),df2dp),1));
   
end
