% [ass,grad,H,sens] = ass_vpl(p,ass_data_fun,ass_pred_fun,model_fun,data,pred)
% 
%   p               parameters
%   ass_data_fun    name of the objective function for the data
%   ass_pred_fun    name of the objective function for the validation
%                   data/predicted value
%   model_fun       name of the function evaluating the model response
%   data            data struct         
%                       data.t
%                       data.y
%                       data.sigma
%   pred            prediction struct
%                       pred.t

function [ass,grad,H,sens] = ass_vpl(p,ass_data_fun,ass_pred_fun,model_fun,data,pred,varargin)

if(nargout<=1)
    assdat = feval(ass_data_fun,p,model_fun,data,varargin{:});
    asspred = feval(ass_pred_fun,p,model_fun,pred,varargin{:});
elseif(nargout==2)
    [assdat,graddat]   = feval(ass_data_fun,p,model_fun,data,varargin{:});
    [asspred,gradpred] = feval(ass_pred_fun,p,model_fun,pred,varargin{:});
elseif(nargout>=3)
    [assdat,graddat,Hdat,sensdat]     = feval(ass_data_fun,p,model_fun,data,varargin{:});
    [asspred,gradpred,Hpred,senspred] = feval(ass_pred_fun,p,model_fun,pred,varargin{:});
end

ass = assdat + asspred;
if(nargout>1)
    grad = graddat + gradpred;
end
if(nargout>2)
    sens = [sensdat;senspred];    
    H = 2*sens'*sens;
end


