% [f,dfdp] = model_fun_ODE1(p,data,varargin)
% 
%   Model function for reactions A->B->C
%   The model function in general calculates the model response/behaviour
%   for given paramters and design, as well as the gradient with respect to
%   parameter changes.
% 
%   data        struct, design
%   p           parameters
% 
%   f           model response
%   dfdp        ndata x npara
%               each row is the gradient w.r. to the parameters

function [f,dfdp] = model_fun_ODE1(p,data,varargin)

x0 = [p(3) 0 0]';

userdata.p = p;

% ode integration parameters
opt = odeset('RelTol',1e-6,'AbsTol',1e-8);
whichobs = data.obs;

if(nargout<=1)
    if(length(data.t)==1)
        [t,x]=ode15s('AtoBtoC',[0,data.t/2,data.t],x0,opt,userdata);
        x = x(3:end,:);
    else
        [t,x]=ode15s('AtoBtoC',data.t,x0,opt,userdata);
    end
elseif(nargout>1)    
    userdata.p = p;
    userdata.xS0 = zeros(length(x0),length(p));
    userdata.xS0(1,3) = 1;
    
    userdata.jacobianfun = 'AtoBtoC_jacobian';
    userdata.dxdkfun = 'AtoBtoC_drdp';
    userdata.rhsfun = 'AtoBtoC';
    
    if(length(data.t)==1)
        [t,x,dxdp]=ode15s_sens('integratefun',[0,data.t/2,data.t],x0,odeset(opt,'Jacobian','on'),userdata);
%         [t,x,dxdp]=ode_sundials_sens('integratefun',[0,data.t/2,data.t],x0,odeset(opt,'Jacobian','on'),userdata);
        x = x(3:end,:);
        dxdp = dxdp(3:end,:,:);
    else
        [t,x,dxdp]=ode15s_sens('integratefun',data.t,x0,odeset(opt,'Jacobian','off'),userdata);
%         [t,x,dxdp]=ode_sundials_sens('integratefun',data.t,x0,odeset(opt,'Jacobian','on'),userdata);
    end
    dfdp = reshape(dxdp(:,whichobs,:),size(dxdp,1)*length(whichobs),size(dxdp,3));
end
f = reshape(x(:,whichobs),size(x,1)*length(whichobs),1);

